from tkinter import *
from tkinter import messagebox
import random
import math
import time
def array(N1, N2=0, N3=0):
    if N2 == 0 and N3==0:
        return [0 for i in range(N1)]
    elif N3 == 0:
        return [[0 for i in range(N2)] for j in range(N1)]
    else:
        return [[[0 for i in range(N3)] for j in range(N2)] for k in range(N1)]
def genMono(canvas, X0, Y0,R):
    RD=R/2
    return canvas.create_oval(X0-RD,Y0-RD, X0+RD, Y0+RD,fill="red")
root=Tk()
root.title("ブラウン運動のシミュレーション")
canvas=Canvas(root, width=200,height=200)
canvas.pack()
Xmax = 10; Ymax = 10; DispR = 1
EPSI = 0.05; SIGMA = 0.25
N=100        # 粒子の数
DT=0.01      # 時間刻み
timerInt=10  # 表示時間間隔
R=array(2,N+1,2); F=array(N+1,2); V=array(2,N+1,2); Bname=array(N+1)
E1 = -48 * EPSI * pow(SIGMA,13)
E2 = 24 * EPSI * pow(SIGMA, 7)
MG = 1; ID1 = 0; ID2 = 1
for i in range(1,N+1):
    F[i][0] = 0; F[i][1] = 0
    V[0][i][0] = 0; V[0][i][1] = 0
    Bname[i] = genMono(canvas, 0, 0, DispR)
i = 0   #分子の生成
while i < N:
    R[0][i + 1][0] = random.random() * (float(Xmax) - 2.0) + 1.0
    R[0][i + 1][1] = random.random() * (Ymax - 2.0) + 1.0
    k = 0; NN = 0
    while True: #便宜的に100回繰り返しても近すぎる座標のとき諦める
      k = k + 1; NN = NN + 1
      dX = R[0][k][0] - R[0][i + 1][0]
      dY = R[0][k][1] - R[0][i + 1][1]
      if dX * dX + dY * dY < 5 or NN > 100: break
    i = i + 1
#■表示
def dsp(ID1):
    global N, canvas, R, Bname
    for i in range(1,N+1):
        X = R[ID1][i][0]*20
        Y = R[ID1][i][1]*20
        canvas.coords(Bname[i], X, Y, X, Y)
#■力の計算
def force(ID1):
    global N, F, R
    for i in range(1,N+1): # 力をランダムな力にする
        F[i][0] = random.random()-0.5
        F[i][1] = random.random()-0.5
#■速度の計算
def velocity(ID1,ID2):
    global N, F, V,MG
    for i in range(1,N+1):
        V[ID2][i][0] = V[ID1][i][0] + DT * F[i][0]/ MG
        V[ID2][i][1] = V[ID1][i][1] + DT * F[i][1]/ MG
    T = 0 # 温度境界条件
    for i in range(1,N+1):
        VX = V[ID2][i][0]; VY = V[ID2][i][1]
        T += VX * VX + VY * VY
    T = math.sqrt(T) * 0.1
    if T > 0.00001:
        for i in range(1, N+1):
            V[ID2][i][0] /= T; V[ID2][i][1] /= T
#■位置の計算
def place(ID1,ID2):
    global N, V, X
    for i in range(1,N+1):
        R[ID2][i][0] = R[ID1][i][0] + DT * V[ID2][i][0]
        R[ID2][i][1] = R[ID1][i][1] + DT * V[ID2][i][1]
#■壁への衝突
def wallHit(ID2):
    for i in range(1,N+1):
        if R[ID2][i][0] < (0.5 * SIGMA):
            R[ID2][i][0] = 0.5 * SIGMA
            V[ID2][i][0] = abs(V[ID2][i][0]) * 0.0  # 0.0 以上にすると端による確率が少なくなる
            
        if R[ID2][i][0] > (Xmax - 0.5 * SIGMA):
            R[ID2][i][0] = Xmax - 0.5 * SIGMA
            V[ID2][i][0] = -abs(V[ID2][i][0]) * 0.0 # 0.0 以上にすると端による確率が少なくなる
            
        if R[ID2][i][1] < (0.5 * SIGMA):
            R[ID2][i][1] = 0.5 * SIGMA
            V[ID2][i][1] = abs(V[ID2][i][1]) * 0.0  # 0.0 以上にすると端による確率が少なくなる
            
        if R[ID2][i][1] > (Ymax - 0.5 * SIGMA):
            R[ID2][i][1] = Ymax - 0.5 * SIGMA
            V[ID2][i][1] = -abs(V[ID2][i][1]) * 0.0 # 0.0 以上にすると端による確率が少なくなる
            
#■シミュレーション
def simulate(ID1, ID2):
    dsp(ID1)
    force(ID1)
    velocity(ID1,ID2)
    place(ID1,ID2)
    wallHit(ID2)
    dsp(ID1)
    root.update_idletasks()
    root.update()
#■画面コピー用マウスイベント  
def leftMouseDown(event):
    global ID1, ID2
    ID2=1-ID1
    simulate(ID1, ID2)
    
    ID1=ID2
#■メイン処理
#左ボタン
#canvas.bind("<Button-1>",leftMouseDown)  #画面コピー用マウス
#画面コピーするときは以下をコメントにする
ID1=0
while True:
    ID2=1-ID1
    simulate(ID1,ID2)
    time.sleep(0.01)
    ID1=ID2
